@@ -67,7 +67,7 @@ class CSimpleLine : public CRenderizableShaderWireFrame
 
 	void onUpdateBuffers_Wireframe() override;
 
-	mrpt::math::TBoundingBox getBoundingBox() const override;
+	mrpt::math::TBoundingBoxf internalBoundingBoxLocal() const override;
 
 	/** Constructor
 	 */
