@@ -60,6 +60,8 @@ void CMesh::updateTriangles() const
 
 	CRenderizable::notifyChange();
 
+	std::unique_lock<std::shared_mutex> lckWrite(m_meshDataMtx.data);
+
 	// Remember:
 	/** List of triangles in the mesh */
 	// mutable
@@ -74,6 +76,8 @@ void CMesh::updateTriangles() const
 	const auto cols = Z.cols();
 	const auto rows = Z.rows();
 
+	m_zMin = m_zMax = 0;
+
 	actualMesh.clear();
 	if (cols == 0 && rows == 0) return;	 // empty mesh
 
@@ -103,6 +107,9 @@ void CMesh::updateTriangles() const
 
 	mrpt::opengl::TTriangle tri;
 
+	m_zMin = std::numeric_limits<float>::max();
+	m_zMax = -std::numeric_limits<float>::max();
+
 	for (int iX = 0; iX < rows - 1; iX++)
 		for (int iY = 0; iY < cols - 1; iY++)
 		{
@@ -191,6 +198,13 @@ void CMesh::updateTriangles() const
 				// Add triangle:
 				actualMesh.emplace_back(tri, tvi);
 
+				// ... and update z bbox:
+				for (int i = 0; i < 3; i++)
+				{
+					mrpt::keep_min(m_zMin, tri.z(i));
+					mrpt::keep_max(m_zMax, tri.z(i));
+				}
+
 				// For averaging normals:
 				for (unsigned long k : tvi.vind)
 				{
@@ -272,6 +286,13 @@ void CMesh::updateTriangles() const
 				// Add triangle:
 				actualMesh.emplace_back(tri, tvi);
 
+				// ... and update z bbox:
+				for (int i = 0; i < 3; i++)
+				{
+					mrpt::keep_min(m_zMin, tri.z(i));
+					mrpt::keep_max(m_zMax, tri.z(i));
+				}
+
 				// For averaging normals:
 				for (unsigned long k : tvi.vind)
 				{
@@ -300,7 +321,7 @@ void CMesh::render(const RenderContext& rc) const
 {
 	switch (rc.shader_id)
 	{
-		case DefaultShaderID::TEXTURED_TRIANGLES:
+		case DefaultShaderID::TEXTURED_TRIANGLES_LIGHT:
 			if (!m_isWireFrame)
 				CRenderizableShaderTexturedTriangles::render(rc);
 			break;
@@ -321,9 +342,14 @@ void CMesh::onUpdateBuffers_Wireframe()
 {
 	auto& vbd = CRenderizableShaderWireFrame::m_vertex_buffer_data;
 	auto& cbd = CRenderizableShaderWireFrame::m_color_buffer_data;
+	std::unique_lock<std::shared_mutex> wfWriteLock(
+		CRenderizableShaderWireFrame::m_wireframeMtx.data);
+
 	vbd.clear();
 	cbd.clear();
 
+	std::shared_lock<std::shared_mutex> lckRead(m_meshDataMtx.data);
+
 	for (auto& i : actualMesh)
 	{
 		const mrpt::opengl::TTriangle& t = i.first;
@@ -340,13 +366,19 @@ void CMesh::onUpdateBuffers_Wireframe()
 			cbd.emplace_back(t.r(k1), t.g(k1), t.b(k1), t.a(k1));
 		}
 	}
+
+	notifyBBoxChange();
 }
 
 void CMesh::onUpdateBuffers_TexturedTriangles()
 {
 	auto& tris = CRenderizableShaderTexturedTriangles::m_triangles;
+	std::unique_lock<std::shared_mutex> writeLock(m_trianglesMtx.data);
+
 	tris.clear();
 
+	std::shared_lock<std::shared_mutex> lckRead(m_meshDataMtx.data);
+
 	for (auto& i : actualMesh)
 	{
 		const mrpt::opengl::TTriangle& t = i.first;
@@ -372,6 +404,8 @@ void CMesh::onUpdateBuffers_TexturedTriangles()
 
 		tris.emplace_back(std::move(tri));
 	}
+
+	notifyBBoxChange();
 }
 
 /*---------------------------------------------------------------
@@ -587,6 +621,8 @@ mrpt::math::TPolygonWithPlane createPolygonFromTriangle(
 void CMesh::updatePolygons() const
 {
 	if (!m_trianglesUpToDate) updateTriangles();
+
+	std::shared_lock<std::shared_mutex> lckRead(m_meshDataMtx.data);
 	size_t N = actualMesh.size();
 	tmpPolys.resize(N);
 	transform(
@@ -596,9 +632,10 @@ void CMesh::updatePolygons() const
 	CRenderizable::notifyChange();
 }
 
-auto CMesh::getBoundingBox() const -> mrpt::math::TBoundingBox
+auto CMesh::internalBoundingBoxLocal() const -> mrpt::math::TBoundingBoxf
 {
-	return trianglesBoundingBox().compose(m_pose);
+	return mrpt::math::TBoundingBoxf::FromUnsortedPoints(
+		{m_xMin, m_yMin, m_zMin}, {m_xMax, m_yMax, m_zMax});
 }
 
 void CMesh::adjustGridToImageAR()
