@@ -30,6 +30,8 @@ constexpr double text_kerning = 0.1;
 
 void CText::onUpdateBuffers_Text()
 {
+	std::unique_lock<std::shared_mutex> writeLock(m_textDataMtx.data);
+
 	auto& vbd = CRenderizableShaderText::m_vertex_buffer_data;
 	auto& tris = CRenderizableShaderText::m_triangles;
 	auto& cbd = CRenderizableShaderText::m_color_buffer_data;
@@ -143,10 +145,11 @@ void CText::serializeFrom(mrpt::serialization::CArchive& in, uint8_t version)
 	};
 }
 
-auto CText::getBoundingBox() const -> mrpt::math::TBoundingBox
+auto CText::internalBoundingBoxLocal() const -> mrpt::math::TBoundingBoxf
 {
-	return mrpt::math::TBoundingBox({0, 0, 0}, {0, 0, 0}).compose(m_pose);
+	return {{0.f, 0.f, 0.f}, {0.f, 0.f, 0.f}};
 }
+
 void CText::toYAMLMap(mrpt::containers::yaml& propertiesMap) const
 {
 	CRenderizable::toYAMLMap(propertiesMap);
