@@ -110,9 +110,4 @@ int main()
 		std::cerr << "MRPT error: " << mrpt::exception_to_str(e) << std::endl;
 		return -1;
 	}
-	catch (...)
-	{
-		printf("Untyped exception!!");
-		return -1;
-	}
 }
